package repository

import (
	"electricity_bill_calc/config"
	"electricity_bill_calc/global"
	"electricity_bill_calc/model"
	"electricity_bill_calc/utils"
	"time"

	"xorm.io/builder"
)

type _ChargeRepository struct{}

var ChargeRepo _ChargeRepository

func (_ChargeRepository) ListPagedChargeRecord(keyword, beginDate, endDate string, page int) ([]model.ChargeWithName, int64, error) {
	var cond = builder.NewCond()
	if len(keyword) != 0 {
		cond = cond.And(builder.Like{"d.name", keyword}.Or(builder.Like{"d.abbr", keyword}))
	}
	if len(beginDate) != 0 {
		beginTime, err := time.ParseInLocation("2006-01-02", beginDate, time.Local)
		beginTime = utils.VeryBeginOfDate(beginTime)
		if err != nil {
			return make([]model.ChargeWithName, 0), 0, err
		}
		cond = cond.And(builder.Gte{"c.created_at": beginTime})
	}
	if len(endDate) != 0 {
		endTime, err := time.ParseInLocation("2006-01-02", endDate, time.Local)
		endTime = utils.VeryEndOfDate(endTime)
		if err != nil {
			return make([]model.ChargeWithName, 0), 0, err
		}
		cond = cond.And(builder.Lte{"c.created_at": endTime})
	}
	startItem := (page - 1) * config.ServiceSettings.ItemsPageSize
	total, err := global.DBConn.
		Alias("d").
		Join("INNER", []string{"user_charge", "c"}, "c.user_id=d.id").
		Where(cond).
		NoAutoCondition().
		Count(&model.ChargeWithName{})
	if err != nil {
		return nil, -1, err
	}
	charges := make([]model.ChargeWithName, 0)
	err = global.DBConn.
		Alias("d").
		Join("INNER", []string{"user_charge", "c"}, "c.user_id=d.id").
		Where(cond).
		Limit(config.ServiceSettings.ItemsPageSize, startItem).
		NoAutoCondition().
		Find(&charges)
	return charges, total, err
}

func (_ChargeRepository) LastValidChargeTo(uid string) (time.Time, error) {
	veryBlankTime, _ := time.Parse("2006-01-02 15:04:05", "0001-01-01 00:00:00")
	var records []string
	err := global.DBConn.
		Table(&model.UserCharge{}).
		Where(builder.Eq{"settled": true, "cancelled": false, "refunded": false, "user_id": uid}).
		Cols("charge_to").
		Find(&records)
	if err != nil {
		return veryBlankTime, nil
	}
	mappedRecords := utils.Map(records, func(elem string) time.Time {
		t, _ := time.Parse(time.RFC3339, elem)
		return utils.VeryBeginOfDate(t)
	})
	lastValid := utils.Reduce(mappedRecords, veryBlankTime, func(acc, elem time.Time) time.Time {
		if elem.After(acc) {
			return elem
		} else {
			return acc
		}
	})
	return lastValid, nil
}
